<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Generate and Import Coupons for Magento 2
 */

namespace Amasty\Xcoupon\Model;

use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\Stdlib\DateTime;
use Magento\SalesRule\Helper\Coupon;
use Magento\SalesRule\Model\Coupon\CodegeneratorInterface;
use Magento\SalesRule\Model\Coupon\Massgenerator as CouponMassgenerator;
use Magento\SalesRule\Model\CouponFactory;

class Massgenerator extends CouponMassgenerator implements
    CodegeneratorInterface
{
    /**
     * @var array
     */
    protected $alphabet = [];

    /**
     * @var array
     */
    protected $counts = [];

    /**
     * @var array
     */
    protected $pos = [];

    /**
     * @var null
     */
    protected $pattern = null;

    public function __construct(
        Context $context,
        Registry $registry,
        Coupon $salesRuleCoupon,
        CouponFactory $couponFactory,
        DateTime\DateTime $date,
        DateTime $dateTime,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        // no "0" and "1" as they are confusing
        $this->alphabet['D'] = [2, 3, 4, 5, 6, 7, 8, 9];
        $this->counts['D'] = count($this->alphabet['D']);
        // no I, Q and O as they are confusing
        $this->alphabet['L'] = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N',
            'P', 'R', 'S', 'T', 'U', 'V', 'W', 'X'];
        $this->counts['L'] = count($this->alphabet['L']);

        $this->pos['L'] = $this->pos['D'] = 0;
        parent::__construct(
            $context,
            $registry,
            $salesRuleCoupon,
            $couponFactory,
            $date,
            $dateTime,
            $resource,
            $resourceCollection,
            $data
        ); // TODO: Change the autogenerated stub
    }

    /**
     * @return string
     * @throws LocalizedException
     */
    public function generateCode()
    {
        $code = '';
        if ($this->getPattern() !== null) {
            // we call rand() per one code, not per one letter
            // as it is a bit faster
            shuffle($this->alphabet['D']);
            shuffle($this->alphabet['L']);

            $pattern = str_split($this->getPattern());
            foreach ($pattern as $i) {
                if (empty($this->alphabet[$i])) {
                    $code .= $i;
                } else {
                    $code .= $this->alphabet[$i][$this->pos[$i]];
                    $this->pos[$i] = ($this->pos[$i] + 1) % $this->counts[$i];
//                    $code .= $this->alphabet[$i][rand(0, $this->counts[$i]-1)];
                }
            }
        } else {
            throw new LocalizedException(__('The pattern must be set up.'));
        }

        return $code;
    }

    public function increaseLength()
    {
        return null;
    }

    /**
     * Validate data input
     *
     * @param array $data
     * @return bool
     */
    public function validateData($data)
    {
        return !empty($data)
            && !empty($data['qty'])
            && !empty($data['rule_id'])
            && !empty($data['pattern']);
    }

    public function validateGenerateCouponsData(array $data): bool
    {
        return !empty($data)
            && !empty($data['qty'])
            && !empty($data['rule_id'])
            && !empty($data['amasty_xcoupon_delete_existing_coupons']);
    }

    public function clean()
    {
        if ($this->getRuleId()) {
            $connection = $this->getResource()->getConnection();
            $tableName = $this->getResource()->getTable('salesrule_coupon');
            $connection->delete($tableName, ['rule_id = ?' => $this->getRuleId()]);
        } else {
            throw new LocalizedException(__('The rule must be set up.'));
        }
    }
}
