<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Generate and Import Coupons for Magento 2
 */

namespace Amasty\Xcoupon\Model\ResourceModel\Report\Usage\Grid;

use Magento\Framework\Data\Collection\Db\FetchStrategyInterface as FetchStrategy;
use Magento\Framework\Data\Collection\EntityFactoryInterface as EntityFactory;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Psr\Log\LoggerInterface as Logger;

class Collection extends \Magento\Sales\Model\ResourceModel\Order\Grid\Collection
{
    public const ID_FIELD_NAME = 'id';

    public function __construct(
        EntityFactory $entityFactory,
        Logger $logger,
        FetchStrategy $fetchStrategy,
        EventManager $eventManager
    ) {
        parent::__construct(
            $entityFactory,
            $logger,
            $fetchStrategy,
            $eventManager
        );
        $this->_setIdFieldName(self::ID_FIELD_NAME);
    }

    protected function _initSelect()
    {
        parent::_initSelect();

        $this->join(
            ['order' => 'sales_order'],
            'order.entity_id=main_table.entity_id',
            ['coupon_code' => 'order.coupon_code']
        );

        $this->getSelect()->joinLeft(
            ['track' => $this->getTable('sales_shipment_track')],
            'track.order_id=main_table.entity_id',
            ['track_number' => 'GROUP_CONCAT(track.track_number)']
        );

        $this->getSelect()->where('order.coupon_code is not NULL');

        $this->getSelect()->group(['main_table.entity_id', 'order.coupon_code']);
    }
}
