<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Generate and Import Coupons for Magento 2
 */

namespace Amasty\Xcoupon\Observer\Controller\Action;

use Magento\Checkout\Model\Session;
use Magento\Framework\App\Request\Http;
use Magento\Framework\Event\Observer;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\SalesRule\Model\CouponFactory;

class Predispatch implements \Magento\Framework\Event\ObserverInterface
{
    public const XCOUPON_COOKIE = 'xcoupon_message';

    /**
     * @var Http
     */
    private $request;

    /**
     * @var Session
     */
    private $checkoutSession;

    /**
     * @var CookieMetadataFactory
     */
    private $cookieMetadataFactory;

    /**
     * @var CookieManagerInterface
     */
    private $cookieManager;

    /**
     * @var ManagerInterface
     */
    private $messageManager;

    /**
     * @var CouponFactory
     */
    private $couponFactory;

    public function __construct(
        Http $request,
        Session $customerSession,
        CookieMetadataFactory $cookieMetadataFactory,
        CookieManagerInterface $cookieManager,
        ManagerInterface $messageManager,
        CouponFactory $couponFactory
    ) {
        $this->request = $request;
        $this->checkoutSession = $customerSession;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->cookieManager = $cookieManager;
        $this->messageManager = $messageManager;
        $this->couponFactory = $couponFactory;
    }

    /**
     * @param Observer $observer
     *
     * @return void
     */
    public function execute(\Magento\Framework\Event\Observer $observer): void
    {
        $couponCode = $this->request->getParam('coupon_code');
        if (!$couponCode) {
            return;
        }

        $cartQuote = $this->checkoutSession->getQuote();
        if ($cartQuote->getCouponCode()) {
            return;
        }

        $cartQuote->setCouponCode($couponCode)->save();

        $coupon = $this->couponFactory->create();
        $coupon->load($couponCode, 'code');
        if ($coupon->getId() && $couponCode === $cartQuote->getCouponCode()) {
            $this->messageManager->addSuccessMessage(__('You used coupon code "%1".', $couponCode));
        } else {
            $this->messageManager->addErrorMessage(__('The coupon code "%1" is not valid.', $couponCode));
        }

        $this->setXcouponCookie();
    }

    private function setXcouponCookie(): void
    {
        $metadata = $this->cookieMetadataFactory->createPublicCookieMetadata();
        $metadata->setPath($this->checkoutSession->getCookiePath())
            ->setSecure(false)
            ->setHttpOnly(false);

        $this->cookieManager->setPublicCookie(
            self::XCOUPON_COOKIE,
            1,
            $metadata
        );
    }
}
